@@ -9,8 +9,8 @@
 namespace hannk {
 
 struct InterpreterOptions {
-    // Whether to dump information during scheduling.
-    bool verbose = false;
+    // Verbosity level. 0 = None.
+    int verbosity = 0;
 
     // Whether to enable tracing.
     bool trace = false;
@@ -18,6 +18,7 @@ struct InterpreterOptions {
 
 class Interpreter {
     std::unique_ptr<OpGroup> model_;
+    std::unique_ptr<char[]> tensor_storage_arena_;
 
     void init(InterpreterOptions options);
 
