@@ -5,25 +5,26 @@
 #include "halide/add_uint8_uint8.h"
 #include "halide/average_pool_uint8.h"
 #include "halide/constants.h"
-#include "halide/conv_uint8.h"
+#include "halide/conv_u8_u8_i16.h"
+#include "halide/conv_u8_u8_u8.h"
 #ifdef CONV_R16
-#include "halide/conv_r16_uint8.h"
+#include "halide/conv_r16_u8_u8_i16.h"
+#include "halide/conv_r16_u8_u8_u8.h"
 #endif
 #include "halide/copy_uint8_uint8.h"
 #include "halide/depthwise_conv_broadcast_uint8.h"
-#include "halide/depthwise_conv_dm1_uint8.h"
+#include "halide/depthwise_conv_shallow_uint8.h"
 #include "halide/depthwise_conv_uint8.h"
 #include "halide/elementwise_5xint16_1xuint8int16.h"
 #include "halide/elementwise_5xuint8_1xuint8.h"
 #include "halide/fill_uint8.h"
-#include "halide/fully_connected_uint8_int16.h"
-#include "halide/fully_connected_uint8_uint8.h"
 #include "halide/l2_normalization_uint8.h"
 #include "halide/max_pool_uint8.h"
 #include "halide/mean_uint8.h"
 #include "halide/mul_uint8_uint8_uint8.h"
 #include "halide/softmax_uint8.h"
 #include "halide/tile_conv_filter_uint8.h"
+#include "halide/upsample_channels_uint8.h"
 #include "interpreter/elementwise_program.h"
 #include "interpreter/ops.h"
 #include "util/error_util.h"
@@ -32,6 +33,19 @@ namespace hannk {
 
 namespace {
 
+#if 0
+// Useful for debugging
+std::string dims_to_string(const halide_buffer_t *buf) {
+    std::ostringstream oss;
+    oss << "{";
+    for (int i = 0; i < buf->dimensions; i++) {
+        oss << "{" << buf->dim[i] << "},";
+    }
+    oss << "}";
+    return oss.str();
+}
+#endif
+
 // Split a dimension d into two new dimensions. Dim d will have min 0
 // and extent factor, while the new dim d + 1 will have the outer split dimension.
 template<typename T>
@@ -49,6 +63,7 @@ void split(int d, int factor, HalideBuffer<T> &buf) {
     dim1.stride *= factor;
 }
 
+// TODO: FuseType::Delete is unused and could likely be removed.
 enum class FuseType {
     // Delete the second of the fused dimension, reducing the rank by 1.
     Delete,
@@ -97,8 +112,8 @@ bool can_fuse_xy(FuseType type, const halide_buffer_t *buf) {
 }
 
 // Fuse dimensions d0 and d1 of buf.
-// If pad is true, add a new dimension of extent 1 and min 0 at the end.
-// If pad is false, d1 is deleted from the buffer.
+// If type==Pad, add a new dimension of extent 1 and min 0 at the end.
+// If type==Delete, d1 is deleted from the buffer.
 void fuse(int d0, int d1, FuseType type, halide_buffer_t *buf) {
     assert(can_fuse(d0, d1, type, buf));
     halide_dimension_t &dim0 = buf->dim[d0];
@@ -595,6 +610,25 @@ TResult implement_binary(BinaryOp::Operator op, TOperand a, TOperand b) {
     }
 }
 
+template<typename TOperand, typename TResult>
+bool try_scalar_binary_op(BinaryOp::Operator op, const TensorPtr &a, const TensorPtr &b, const TensorPtr &result) {
+    if (a->type() == halide_type_of<TOperand>() &&
+        b->type() == halide_type_of<TOperand>() &&
+        result->type() == halide_type_of<TResult>()) {
+        const auto &a_buf = a->buffer<const TOperand>();
+        const auto &b_buf = b->buffer<const TOperand>();
+        const auto &result_buf = result->buffer<TResult>();
+
+        // This is really slow, only intended to support scalar operations.
+        const auto scalar_op = [&](TOperand a_scalar, TOperand b_scalar, TResult &result_scalar) {
+            result_scalar = implement_binary<TResult, TOperand>(op, a_scalar, b_scalar);
+        };
+        scalar_elementwise_loop_nest(scalar_op, a_buf, b_buf, result_buf);
+        return true;
+    }
+    return false;
+}
+
 }  // namespace
 
 void BinaryOp::execute() {
@@ -620,39 +654,45 @@ void BinaryOp::execute() {
         default:
             break;
         }
-    } else if (in1->type() == halide_type_of<int32_t>() &&
-               in2->type() == halide_type_of<int32_t>() &&
-               out->type() == halide_type_of<int32_t>()) {
-        const auto &in1_buf = in1->buffer<const int32_t>();
-        const auto &in2_buf = in2->buffer<const int32_t>();
-        const auto &out_buf = out->buffer<int32_t>();
-
+    } else {
         // This is really slow, only intended to support scalar operations.
-        auto scalar_op = [&](int32_t a, int32_t b, int32_t &result) {
-            result = implement_binary<int32_t>(op_, a, b);
-        };
-        scalar_elementwise_loop_nest(scalar_op, in1_buf, in2_buf, out_buf);
-        return;
-    } else if (out->type() == halide_type_of<bool>() && out->rank() == 0) {
-        double in1_scalar = dequantize_scalar(in1.get());
-        double in2_scalar = dequantize_scalar(in2.get());
-        auto out_buf = out->buffer<bool>();
+        if (try_scalar_binary_op<int32_t, int32_t>(op_, in1, in2, out)) {
+            return;
+        }
 
-        switch (op_) {
-        case Less:
-            out_buf() = in1_scalar < in2_scalar;
-            return;
-        case LessEqual:
-            out_buf() = in1_scalar <= in2_scalar;
-            return;
-        case Equal:
-            out_buf() = in1_scalar == in2_scalar;
-            return;
-        case NotEqual:
-            out_buf() = in1_scalar != in2_scalar;
-            return;
-        default:
-            break;
+        // TODO: these can be useful for debugging pipelines that use op variants we don't fully support yet
+        // (e.g. float32) -- leaving this here (but commented out) as a useful reference, but *please*
+        // don't add any permanent usage here at this time (the ops almost certainly need to be written in Halide).
+        //
+        // if (try_scalar_binary_op<float, float>(op_, in1, in2, out)) {
+        //     return;
+        // }
+        // // This is for the LESS, etc operators, which may store results in uint8 rather than bool
+        // if (try_scalar_binary_op<float, uint8_t>(op_, in1, in2, out)) {
+        //     return;
+        // }
+
+        if (out->type() == halide_type_of<bool>() && out->rank() == 0) {
+            double in1_scalar = dequantize_scalar(in1.get());
+            double in2_scalar = dequantize_scalar(in2.get());
+            auto out_buf = out->buffer<bool>();
+
+            switch (op_) {
+            case Less:
+                out_buf() = in1_scalar < in2_scalar;
+                return;
+            case LessEqual:
+                out_buf() = in1_scalar <= in2_scalar;
+                return;
+            case Equal:
+                out_buf() = in1_scalar == in2_scalar;
+                return;
+            case NotEqual:
+                out_buf() = in1_scalar != in2_scalar;
+                return;
+            default:
+                break;
+            }
         }
     }
     HLOG(FATAL)
@@ -692,10 +732,14 @@ void ConcatenationOp::execute() {
     }
 }
 
-halide_type_t Conv2DOp::filter_type() const {
+halide_type_t ConvOp::filter_type() const {
     if (input()->type() == halide_type_of<uint8_t>() &&
         output()->type() == halide_type_of<uint8_t>()) {
-        const halide_filter_metadata_t *metadata = conv_uint8_metadata();
+        const halide_filter_metadata_t *metadata = conv_u8_u8_u8_metadata();
+        return metadata->arguments[2].type;
+    } else if (input()->type() == halide_type_of<uint8_t>() &&
+               output()->type() == halide_type_of<int16_t>()) {
+        const halide_filter_metadata_t *metadata = conv_u8_u8_i16_metadata();
         return metadata->arguments[2].type;
     } else {
         HLOG(FATAL) << "Unsupported type " << output()->type() << "\n";
@@ -703,79 +747,83 @@ halide_type_t Conv2DOp::filter_type() const {
     }
 }
 
-BoundsMap Conv2DOp::map_bounds(int input_idx, int output_idx) const {
+BoundsMap ConvOp::map_bounds(int input_idx, int output_idx) const {
 #ifdef CONV_R16
     const int unroll_reduction = filter()->extent(0) >= 16 ? 16 : 4;
 #else
     const int unroll_reduction = 4;
 #endif
     if (input_idx == 0) {
-        return BoundsMap(4, output()->rank())
+        BoundsMap result(input()->rank(), output()->rank());
+        result
             .constant(0, align_up(input()->extent(0), unroll_reduction))
-            .downsample(1, 1, stride_[0], Interval(0, dilation_[0] * (filter()->extent(1) - 1)))
-            .downsample(2, 2, stride_[1], Interval(0, dilation_[1] * (filter()->extent(2) - 1)))
-            .elementwise(3, 3);
+            .elementwise(input()->rank() - 1, input()->rank() - 1);
+        for (int i = 1; i < input()->rank() - 1; i++) {
+            result.downsample(i, i, stride_[i - 1], Interval(0, dilation_[i - 1] * (filter()->extent(i) - 1)));
+        }
+        return result;
     } else if (input_idx == 1) {
         // Pass minimal sized buffers to learn about the alignment requirements.
         HalideBuffer<uint8_t> input_buf(nullptr, 1, 1, 1, 1);
         HalideBuffer<int32_t> bias_buf(nullptr, 1);
-        HalideBuffer<void> filter_buf(filter_type(), 1, 1, 1, 1, 1, 1);
-        // TODO: How to initialize the above buffer without allocating?
-        filter_buf.deallocate();
-        HalideBuffer<uint8_t> output_buf;
-        conv_uint8(input_buf, 0, filter_buf, 0, bias_buf, 1, 1, 1, 1, 0, 0, 0, 0, 0, output_buf);
+        HalideBuffer<void> filter_buf(filter_type(), nullptr, 1, 1, 1, 1, 1, 1);
+        HalideBuffer<uint8_t> output_buf(nullptr, 1, 1, 1, 1);
+        conv_u8_u8_u8(input_buf, 0, filter_buf, 0, bias_buf, 1, 1, 1, 1, 0, 0, 0, 0, 0, output_buf);
 
         const int vector_reduction = filter_buf.dim(0).extent();
         const int vector_tile = filter_buf.dim(1).extent();
         const int channel_alignment = unroll_reduction / vector_reduction;
-        return BoundsMap(6, 4)
+        BoundsMap result(input()->rank() + 2, output()->rank());
+        result
             .constant(0, vector_reduction)
             .constant(1, vector_tile)
             .constant(2, align_up(ceil_div(filter()->extent(0), vector_reduction), channel_alignment))
-            .upsample(3, 0, vector_tile)
-            .constant(4, filter()->bounds(1))
-            .constant(5, filter()->bounds(2));
+            .upsample(3, 0, vector_tile);
+        for (int i = 1; i < output()->rank() - 1; i++) {
+            result.constant(i + 3, filter()->bounds(i));
+        }
+        return result;
     } else {
         assert(input_idx == 2);
-        return BoundsMap(1, 4).elementwise(0, 0);
+        return BoundsMap(1, output()->rank()).elementwise(0, 0);
     }
 }
 
 namespace {
 
-void conv_uint8(halide_buffer_t *input, halide_buffer_t *filter, halide_buffer_t *bias,
-                const MultiplyParams &params, const std::array<int, 2> &stride,
-                const std::array<int, 2> &dilation, const Interval &output_range,
-                halide_buffer_t *output) {
-    assert(params.c.exponent() <= 0);
+void call_conv2d(halide_buffer_t *input, halide_buffer_t *filter, halide_buffer_t *bias,
+                 const MultiplyParams &params, const std::array<int, 2> &stride,
+                 const std::array<int, 2> &dilation, const Interval &output_range,
+                 halide_buffer_t *output) {
+    using Conv2DFn = decltype(&::hannk::conv_u8_u8_u8);
+
+    Conv2DFn fn;
 #ifdef CONV_R16
     if (input->dim[0].extent >= 16) {
         // For large reductions, use the big reduction version.
         // TODO: We really ought to be able to do this with GuardWithIf
         // and/or specialize.
-        conv_r16_uint8(input, (uint8_t)params.a_zero, filter, (uint8_t)params.b_zero, bias,
-                       stride[0], stride[1], dilation[0], dilation[1], params.c.mantissa(),
-                       -params.c.exponent(), (uint8_t)params.c_zero, output_range.min, output_range.max,
-                       output);
+        fn = output->type == halide_type_of<int16_t>() ? hannk::conv_r16_u8_u8_i16 : hannk::conv_r16_u8_u8_u8;
     } else
 #endif
     {
-        ::hannk::conv_uint8(input, (uint8_t)params.a_zero, filter, (uint8_t)params.b_zero, bias,
-                            stride[0], stride[1], dilation[0], dilation[1], params.c.mantissa(),
-                            -params.c.exponent(), (uint8_t)params.c_zero, output_range.min, output_range.max,
-                            output);
+        fn = output->type == halide_type_of<int16_t>() ? hannk::conv_u8_u8_i16 : hannk::conv_u8_u8_u8;
     }
+    fn(input, (uint8_t)params.a_zero, filter, (uint8_t)params.b_zero, bias,
+       stride[0], stride[1], dilation[0], dilation[1], params.c.mantissa(),
+       -params.c.exponent(), (uint8_t)params.c_zero, output_range.min, output_range.max,
+       output);
 }
 
 }  // namespace
 
-void Conv2DOp::execute() {
+void ConvOp::execute() {
     const TensorPtr &in = input();
     const TensorPtr &filt = filter();
     const TensorPtr &out = output();
 
     if (in->type() == halide_type_of<uint8_t>() &&
-        out->type() == halide_type_of<uint8_t>()) {
+        (out->type() == halide_type_of<uint8_t>() || out->type() == halide_type_of<int16_t>())) {
         auto input_buf = in->buffer();
         auto filter_buf = filt->buffer();
         auto bias_buf = bias()->buffer();
@@ -786,6 +834,13 @@ void Conv2DOp::execute() {
 
         const auto output_range = get_output_range(activation_, out->quantization());
 
+        // Pad with dummy dimensions up to 2D.
+        while (input_buf.dimensions() < 4) {
+            input_buf.embed(input_buf.dimensions() - 1, 1);
+            output_buf.embed(output_buf.dimensions() - 1, 1);
+            filter_buf.add_dimension();
+        }
+
         assert(filter_buf.dimensions() == 6);
         const int filter_width = filter_buf.dim(4).extent();
         const int filter_height = filter_buf.dim(5).extent();
@@ -794,14 +849,23 @@ void Conv2DOp::execute() {
             // small output sizes.
             // TODO: Maybe we can just treat all of x, y, b as batch dimensions and fuse
             // them all where possible, which might be a further improvement.
-            while (can_fuse_xy(FuseType::Pad, input_buf) && can_fuse_xy(FuseType::Pad, output_buf) &&
+            while (can_fuse_xy(FuseType::Pad, input_buf) &&
+                   can_fuse_xy(FuseType::Pad, output_buf) &&
                    input_buf.dim(1).extent() == output_buf.dim(1).extent()) {
                 fuse_xy(FuseType::Pad, input_buf);
                 fuse_xy(FuseType::Pad, output_buf);
             }
+
+            if (output_buf.dim(1).extent() < output_buf.dim(2).extent()) {
+                // Some networks have shapes with very small x and large y that we can't fuse.
+                // This case is bad for us because we tile the x dimension. It would be better
+                // if we tiled y instead. We can do this by just swapping the x and y dimensions.
+                input_buf.transpose(1, 2);
+                output_buf.transpose(1, 2);
+            }
         }
 
-        conv_uint8(input_buf, filter_buf, bias_buf, params, stride_, dilation_, output_range, output_buf);
+        call_conv2d(input_buf, filter_buf, bias_buf, params, stride_, dilation_, output_range, output_buf);
     } else {
         HLOG(FATAL) << "Unsupported type " << out->type() << "\n";
     }
@@ -810,29 +874,48 @@ void Conv2DOp::execute() {
 namespace {
 
 // Wrapper to dispatch to the appropriate variant of depthwise_conv.
-void depthwise_conv_uint8(
+void call_depthwise_conv_uint8(
     halide_buffer_t *input, halide_buffer_t *filter, halide_buffer_t *bias,
-    int depth_multiplier, const MultiplyParams &params, const std::array<int, 2> &stride, const std::array<int, 2> &dilation,
-    const Interval &output_range, halide_buffer_t *output) {
-    assert(params.c.exponent() <= 0);
-    if (depth_multiplier >= output->dim[0].extent) {
-        depthwise_conv_broadcast_uint8(
-            input, (uint8_t)params.a_zero, filter, (uint8_t)params.b_zero, bias, depth_multiplier,
-            stride[0], stride[1], dilation[0], dilation[1], params.c.mantissa(), -params.c.exponent(),
+    const MultiplyParams &params, const std::array<int, 2> &stride, const std::array<int, 2> &dilation,
+    int input_stride_x, const Interval &output_range, halide_buffer_t *output) {
+    if (input_stride_x != 0) {
+        depthwise_conv_shallow_uint8(
+            input, (uint8_t)params.a_zero, filter, (uint8_t)params.b_zero, bias,
+            stride[0], stride[1], dilation[0], dilation[1], input_stride_x, params.c.mantissa(), -params.c.exponent(),
             (uint8_t)params.c_zero, (uint8_t)output_range.min, (uint8_t)output_range.max, output);
-    } else if (depth_multiplier == 1) {
-        depthwise_conv_dm1_uint8(
-            input, (uint8_t)params.a_zero, filter, (uint8_t)params.b_zero, bias, depth_multiplier,
-            stride[0], stride[1], dilation[0], dilation[1], params.c.mantissa(), -params.c.exponent(),
+    } else if (input->dim[0].extent == 1) {
+        depthwise_conv_broadcast_uint8(
+            input, (uint8_t)params.a_zero, filter, (uint8_t)params.b_zero, bias,
+            stride[0], stride[1], dilation[0], dilation[1], input_stride_x, params.c.mantissa(), -params.c.exponent(),
             (uint8_t)params.c_zero, (uint8_t)output_range.min, (uint8_t)output_range.max, output);
     } else {
         ::hannk::depthwise_conv_uint8(
-            input, (uint8_t)params.a_zero, filter, (uint8_t)params.b_zero, bias, depth_multiplier,
-            stride[0], stride[1], dilation[0], dilation[1], params.c.mantissa(), -params.c.exponent(),
+            input, (uint8_t)params.a_zero, filter, (uint8_t)params.b_zero, bias,
+            stride[0], stride[1], dilation[0], dilation[1], input_stride_x, params.c.mantissa(), -params.c.exponent(),
             (uint8_t)params.c_zero, (uint8_t)output_range.min, (uint8_t)output_range.max, output);
     }
 }
 
+// TODO: this could probably be cached rather than recalculated each time
+int get_depthwise_conv_channel_alignment() {
+    // Pass minimal sized buffers to learn about the alignment requirements.
+    HalideBuffer<uint8_t> input_buf(nullptr, 1, 1, 1, 1);
+    HalideBuffer<int32_t> bias_buf(nullptr, 1);
+    HalideBuffer<uint8_t> filter_buf(nullptr, 1, 1, 1);
+    HalideBuffer<uint8_t> output_buf(nullptr, 1, 1, 1, 1);
+    depthwise_conv_uint8(input_buf, 0, filter_buf, 0, bias_buf, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, output_buf);
+    return input_buf.dim(0).extent();
+}
+
+bool can_be_shallow(int alignment, int extent_0, int extent_1) {
+    // This is correct: we want to use shallow when the vector size (ie, alignment)
+    // is evenly divisble by the number of channels (ie, extent(0)).
+    //
+    // To avoid OOB access for tiny buffers, we also check that the fused width
+    // is at least one vector wide.
+    return (alignment % extent_0) == 0 && (extent_0 * extent_1 >= alignment);
+}
+
 }  // namespace
 
 BoundsMap DepthwiseConv2DOp::map_bounds(int input_idx, int output_idx) const {
@@ -845,13 +928,13 @@ BoundsMap DepthwiseConv2DOp::map_bounds(int input_idx, int output_idx) const {
             .downsample(2, 2, stride_[1], Interval(0, dilation_[1] * (filter()->extent(2) - 1)))
             .elementwise(3, 3);
         if (depth_multiplier_ == 1) {
-            // Pass minimal sized buffers to learn about the alignment requirements.
-            HalideBuffer<uint8_t> input_buf(nullptr, 1, 1, 1, 1);
-            HalideBuffer<int32_t> bias_buf(nullptr, 1);
-            HalideBuffer<uint8_t> filter_buf(nullptr, 1, 1, 1);
-            HalideBuffer<uint8_t> output_buf(nullptr, 1, 1, 1, 1);
-            depthwise_conv_dm1_uint8(input_buf, 0, filter_buf, 0, bias_buf, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, output_buf);
-            result.align_input(0, input_buf.dim(0).extent());
+            const int alignment = get_depthwise_conv_channel_alignment();
+            if (stride_[0] == 1 &&
+                can_be_shallow(alignment, input()->extent(0), input()->extent(1))) {
+                // We can use the shallow version of depthwise here.
+            } else {
+                result.align_input(0, alignment);
+            }
         }
         return result;
     } else if (input_idx == 1) {
@@ -884,8 +967,23 @@ void DepthwiseConv2DOp::execute() {
 
         const auto output_range = get_output_range(activation_, out->quantization());
 
-        depthwise_conv_uint8(input_buf, filter_buf, bias_buf, depth_multiplier_, params,
-                             stride_, dilation_, output_range, output_buf);
+        // If the number of channels is small and divides the channel alignment,
+        // and the stride of the filter in x is 1, we can use the "shallow"
+        // version of depthwise conv, which fuses c and x, and passes the stride
+        // of x into the pipeline manually.
+        int input_stride_x = 0;
+        if (stride_[0] == 1 &&
+            can_fuse_cx(FuseType::InPlace, input_buf) &&
+            can_fuse_cx(FuseType::InPlace, output_buf) &&
+            can_be_shallow(get_depthwise_conv_channel_alignment(), input_buf.dim(0).extent(), input_buf.dim(1).extent())) {
+            input_stride_x = input_buf.dim(1).stride();
+            fuse_cx(FuseType::InPlace, input_buf);
+            fuse_cx(FuseType::InPlace, output_buf);
+        }
+
+        assert(depth_multiplier_ == 1 || depth_multiplier_ >= out->extent(0));
+        call_depthwise_conv_uint8(input_buf, filter_buf, bias_buf, params,
+                                  stride_, dilation_, input_stride_x, output_range, output_buf);
     } else {
         HLOG(FATAL) << "Unsupported type " << out->type() << "\n";
     }
@@ -966,59 +1064,6 @@ void ElementwiseProgramOp::execute() {
     HLOG(FATAL) << "Unsupported elementwise program\n";
 }
 
-BoundsMap FullyConnectedOp::map_bounds(int input_idx, int output_idx) const {
-    assert(output_idx == 0);
-    if (input_idx == 0) {
-        return BoundsMap(2, 2).constant(0, input()->extent(0)).elementwise(1, 1);
-    } else if (input_idx == 1) {
-        return BoundsMap(2, 2).constant(0, filter()->extent(0)).elementwise(1, 0);
-    } else if (input_idx == 2) {
-        return BoundsMap(1, 2).elementwise(0, 0);
-    } else {
-        return BoundsMap(0, 2);
-    }
-}
-
-void FullyConnectedOp::execute() {
-    const TensorPtr &in = input();
-    const TensorPtr &filt = filter();
-    const TensorPtr &out = output();
-
-    if (in->type() == halide_type_of<uint8_t>() &&
-        filt->type() == halide_type_of<uint8_t>()) {
-        auto input_buf = in->buffer();
-        auto filter_buf = filt->buffer();
-        auto bias_buf = bias()->buffer();
-        auto output_buf = out->buffer();
-        // TODO: This should be handled explicitly with a reshape.
-        // It's annoying tflite doesn't require this. This means
-        // that we can't arbitrarily insert padding of the strides
-        // for tensors consumed by this op.
-        while (input_buf.dimensions() > 2) {
-            fuse_cx(FuseType::Delete, input_buf);
-        }
-
-        MultiplyParams params =
-            get_quantized_multiply_params(in->quantization(), filt->quantization(), out->quantization());
-
-        if (out->type() == halide_type_of<uint8_t>()) {
-            const auto output_range = get_output_range(activation_, out->quantization());
-
-            fully_connected_uint8_uint8(
-                input_buf, (uint8_t)params.a_zero, filter_buf, (uint8_t)params.b_zero, bias_buf,
-                (uint8_t)params.c_zero, params.c.mantissa(), -params.c.exponent(), (uint8_t)output_range.min,
-                (uint8_t)output_range.max, output_buf);
-            return;
-        } else if (out->type() == halide_type_of<int16_t>()) {
-            fully_connected_uint8_int16(
-                input_buf, (uint8_t)params.a_zero, filter_buf, (uint8_t)params.b_zero, bias_buf,
-                0, params.c.mantissa(), -params.c.exponent(), 0, 0, output_buf);
-            return;
-        }
-    }
-    HLOG(FATAL) << "Unsupported type " << out->type() << "\n";
-}
-
 BoundsMap GatherOp::map_bounds(int input_idx, int output_idx) const {
     if (input_idx == 0) {
         BoundsMap result = BoundsMap::elementwise(output()->rank());
@@ -1038,15 +1083,46 @@ void GatherOp::execute() {
     HalideBuffer<const int32_t> indices = input(1)->buffer();
     const HalideBuffer<void> &out = output()->buffer();
 
+    // Haven't yet found an instance of TFLite's Gather op that
+    // specifies a nonzero values. Implement and test once we do.
+    HCHECK(batch_dims_ == 0) << "TODO: GatherOp doesn't yet support batch_dim != 0";
+
     if (indices.dimensions() == 0) {
-        indices.embed(0, 0);
+        // Yes, a 0D (scalar) here is documented as legal in TFLite
+        indices.embed(0, 0);  // make 1-D
     }
 
-    for (int i = out.dim(axis_).min(); i <= out.dim(axis_).max(); i++) {
-        HalideBuffer<const void> in_i = in.sliced(axis_, indices(i));
-        HalideBuffer<void> out_i = out.sliced(axis_, i);
-        out_i.copy_from(in_i);
+#ifndef NDEBUG
+    assert(out.dimensions() == in.dimensions() + indices.dimensions() - 1);
+    for (int i = 0; i < out.dimensions(); i++) {
+        if (i < axis_) {
+            assert(out.dim(i).extent() == in.dim(i).extent());
+        } else if (i < axis_ + indices.dimensions()) {
+            assert(out.dim(i).extent() == indices.dim(i - axis_).extent());
+        } else {
+            assert(out.dim(i).extent() == in.dim(i - indices.dimensions() + 1).extent());
+        }
     }
+
+    // Negative values for axis_ are handled by the parser
+    assert(axis_ >= 0 && axis_ < in.dimensions());
+#endif
+
+    // Note that the TFLite Gather op restricts indices to 1D (or 0D),
+    // but other NN libraries (eg NNAPI) alloe for it to be multidimensional,
+    // so we must copy more robustly.
+    //
+    // (TODO: support TFLite's GatherNd op, which is similar to this.)
+    const int pos_dims = indices.dimensions();
+    indices.for_each_element([&](const int *pos) {
+        const int index = indices(pos);
+        HalideBuffer<const void> in_i = in.sliced(axis_, index);
+        HalideBuffer<void> out_i = out.sliced(axis_, pos[0]);
+        for (int i = 1; i < pos_dims; i++) {
+            out_i = out_i.sliced(axis_, pos[i]);
+        }
+        out_i.copy_from(in_i);
+    });
 }
 
 BoundsMap L2NormalizationOp::map_bounds(int input_idx, int output_idx) const {
@@ -1061,10 +1137,23 @@ void L2NormalizationOp::execute() {
     const TensorPtr &in = input();
     const TensorPtr &out = output();
 
+    // Negative values for axis_ must be normalized by the parser
+    assert(axis_ >= 0 && axis_ < in->rank());
+
     if (in->type() == halide_type_of<uint8_t>() &&
         out->type() == halide_type_of<uint8_t>()) {
-        const auto &in_buf = in->buffer();
-        const auto &out_buf = out->buffer();
+        // Make local copies in case we need to transpose them
+        HalideBuffer<void> in_buf = in->buffer();
+        HalideBuffer<void> out_buf = out->buffer();
+
+        // TODO: we currently assume that the axis-is-0 case is the most common
+        // and most important, and optimize for it; the other cases, we just transpose,
+        // which currently requires less-efficient specializations in the Halide code.
+        // Revisit if this proves too slow in practice.
+        if (axis_ != 0) {
+            in_buf.transpose(0, axis_);
+            out_buf.transpose(0, axis_);
+        }
 
         const int input_zero = in->quantization().uniform_zero();
         assert(input_zero >= 0 && input_zero <= 255);
@@ -1120,7 +1209,7 @@ void PadOp::execute() {
             assert(before_padding >= 0 && after_padding >= 0);
             new_shape[d].max += before_padding + after_padding;
         }
-        out->resize(new_shape);
+        out->resize_dynamic(new_shape);
     }
 
     if (out->type().bytes() == 1) {
@@ -1362,7 +1451,7 @@ void ReshapeOp::execute() {
         for (int i : new_shape) {
             b.emplace_back(0, i - 1);
         }
-        out->resize(b);
+        out->resize_dynamic(b);
     }
 
     const auto &input_buf = in->buffer();
@@ -1373,12 +1462,17 @@ void ReshapeOp::execute() {
         assert(new_shape.at(d) == output_buf.dim(d).extent());
     }
 
+    // TODO: we must verify these match (and fail at runtime if not).
+    // That said, we should be able to predict this at parse time
+    // (for non-dynamic tensors) and skip the runtime check most of the time.
     assert(input_buf.number_of_elements() == output_buf.number_of_elements());
+
+    assert(in->is_dense());
+    assert(out->is_dense());
     if (is_alias(input_buf.raw_buffer(), output_buf.raw_buffer())) {
         assert(input_buf.begin() == output_buf.begin());
         assert(input_buf.end() == output_buf.end());
     } else {
-        // TODO: This should also check the strides are dense.
         size_t output_size = output_buf.number_of_elements() * out->type().bytes();
         memcpy(output_buf.data(), input_buf.data(), output_size);
     }
@@ -1500,8 +1594,7 @@ void SpaceDepthOp::execute() {
     const TensorPtr &in = input();
     const TensorPtr &out = output();
 
-    if (in->type() == halide_type_of<uint8_t>() &&
-        out->type() == halide_type_of<uint8_t>()) {
+    if (in->type() == out->type()) {
         const auto &in_buf = in->buffer();
         const auto &out_buf = out->buffer();
 
@@ -1510,6 +1603,8 @@ void SpaceDepthOp::execute() {
         } else {
             DepthToSpace(in_buf, -block_size_, out_buf);
         }
+    } else {
+        HLOG(FATAL) << "Unsupported types " << in->type() << " " << out->type() << "\n";
     }
 }
 
@@ -1565,6 +1660,11 @@ void TileConvFilterOp::execute() {
         int input_zero = in->quantization().uniform_zero();
         int output_zero = out->quantization().uniform_zero();
 
+        while (input_buf.dimensions() < 4) {
+            input_buf.embed(input_buf.dimensions() - 1, 0);
+            output_buf.add_dimension();
+        }
+
         tile_conv_filter_uint8(input_buf, input_zero, output_zero, output_buf);
     } else {
         HLOG(FATAL) << "Unsupported type " << in->type() << "\n";
@@ -1694,6 +1794,28 @@ void UnaryOp::execute() {
         << " for types " << in->type() << ", " << out->type();
 }
 
+BoundsMap UpsampleChannelsOp::map_bounds(int input_idx, int output_idx) const {
+    assert(input_idx == 0);
+    assert(output_idx == 0);
+    int rank = output(output_idx)->rank();
+    assert(rank == input(input_idx)->rank());
+    return BoundsMap::elementwise(rank).upsample(0, 0, factor_);
+}
+
+void UpsampleChannelsOp::execute() {
+    const TensorPtr &in = input();
+    const TensorPtr &out = output();
+
+    if (in->type() == halide_type_of<uint8_t>() && out->type() == halide_type_of<uint8_t>()) {
+        auto in_buf = in->buffer();
+        auto out_buf = out->buffer();
+        upsample_channels_uint8(in_buf, factor_, out_buf);
+        return;
+    }
+    HLOG(FATAL)
+        << "Unsupported UpsampleChannels op for types " << in->type() << ", " << out->type();
+}
+
 void BinaryOp::accept(OpVisitor *v) {
     v->visit(this);
 }
@@ -1702,7 +1824,7 @@ void ConcatenationOp::accept(OpVisitor *v) {
     v->visit(this);
 }
 
-void Conv2DOp::accept(OpVisitor *v) {
+void ConvOp::accept(OpVisitor *v) {
     v->visit(this);
 }
 
@@ -1714,10 +1836,6 @@ void ElementwiseProgramOp::accept(OpVisitor *v) {
     v->visit(this);
 }
 
-void FullyConnectedOp::accept(OpVisitor *v) {
-    v->visit(this);
-}
-
 void GatherOp::accept(OpVisitor *v) {
     v->visit(this);
 }
@@ -1766,6 +1884,10 @@ void TransposeOp::accept(OpVisitor *v) {
     v->visit(this);
 }
 
+void UpsampleChannelsOp::accept(OpVisitor *v) {
+    v->visit(this);
+}
+
 void UnaryOp::accept(OpVisitor *v) {
     v->visit(this);
 }
