@@ -337,8 +337,9 @@ struct Provide : public StmtNode<Provide> {
     std::string name;
     std::vector<Expr> values;
     std::vector<Expr> args;
+    Expr predicate;
 
-    static Stmt make(const std::string &name, const std::vector<Expr> &values, const std::vector<Expr> &args);
+    static Stmt make(const std::string &name, const std::vector<Expr> &values, const std::vector<Expr> &args, const Expr &predicate);
 
     static const IRNodeType _node_type = IRNodeType::Provide;
 };
@@ -412,12 +413,13 @@ struct Realize : public StmtNode<Realize> {
     static const IRNodeType _node_type = IRNodeType::Realize;
 };
 
-/** A sequence of statements to be executed in-order. 'rest' may be
- * undefined. Used rest.defined() to find out. */
+/** A sequence of statements to be executed in-order. 'first' is never
+    a Block, so this can be treated as a linked list. */
 struct Block : public StmtNode<Block> {
     Stmt first, rest;
 
     static Stmt make(Stmt first, Stmt rest);
+
     /** Construct zero or more Blocks to invoke a list of statements in order.
      * This method may not return a Block statement if stmts.size() <= 1. */
     static Stmt make(const std::vector<Stmt> &stmts);
@@ -529,7 +531,6 @@ struct Call : public ExprNode<Call> {
         mul_shift_right,
         mux,
         popcount,
-        predicate,
         prefetch,
         promise_clamped,
         random,
@@ -651,6 +652,19 @@ struct Call : public ExprNode<Call> {
         return is_intrinsic() && this->name == get_intrinsic_name(op);
     }
 
+    bool is_intrinsic(std::initializer_list<IntrinsicOp> intrinsics) const {
+        for (IntrinsicOp i : intrinsics) {
+            if (is_intrinsic(i)) {
+                return true;
+            }
+        }
+        return false;
+    }
+
+    bool is_tag() const {
+        return is_intrinsic({Call::likely, Call::likely_if_innermost, Call::strict_float});
+    }
+
     /** Returns a pointer to a call node if the expression is a call to
      * one of the requested intrinsics. */
     static const Call *as_intrinsic(const Expr &e, std::initializer_list<IntrinsicOp> intrinsics) {
@@ -665,7 +679,7 @@ struct Call : public ExprNode<Call> {
     }
 
     static const Call *as_tag(const Expr &e) {
-        return as_intrinsic(e, {Call::likely, Call::likely_if_innermost, Call::predicate, Call::strict_float});
+        return as_intrinsic(e, {Call::likely, Call::likely_if_innermost, Call::strict_float});
     }
 
     bool is_extern() const {
