@@ -16,6 +16,7 @@
 #include "BoundsInference.h"
 #include "CSE.h"
 #include "CanonicalizeGPUVars.h"
+#include "ClampUnsafeAccesses.h"
 #include "CompilerLogger.h"
 #include "Debug.h"
 #include "DebugArguments.h"
@@ -76,7 +77,6 @@
 namespace Halide {
 namespace Internal {
 
-using std::map;
 using std::ostringstream;
 using std::string;
 using std::vector;
@@ -109,15 +109,8 @@ void lower_impl(const vector<Function> &output_funcs,
                 Module &result_module) {
     auto time_start = std::chrono::high_resolution_clock::now();
 
-    // Compute an environment
-    map<string, Function> env;
-    for (const Function &f : output_funcs) {
-        populate_environment(f, env);
-    }
-
     // Create a deep-copy of the entire graph of Funcs.
-    vector<Function> outputs;
-    std::tie(outputs, env) = deep_copy(output_funcs, env);
+    auto [outputs, env] = deep_copy(output_funcs, build_environment(output_funcs));
 
     bool any_strict_float = strictify_float(env, t);
     result_module.set_any_strict_float(any_strict_float);
@@ -137,9 +130,7 @@ void lower_impl(const vector<Function> &output_funcs,
 
     // Compute a realization order and determine group of functions which loops
     // are to be fused together
-    vector<string> order;
-    vector<vector<string>> fused_groups;
-    std::tie(order, fused_groups) = realization_order(outputs, env);
+    auto [order, fused_groups] = realization_order(outputs, env);
 
     // Try to simplify the RHS/LHS of a function definition by propagating its
     // specializations' conditions
@@ -172,6 +163,12 @@ void lower_impl(const vector<Function> &output_funcs,
     debug(1) << "Computing bounds of each function's value\n";
     FuncValueBounds func_bounds = compute_function_value_bounds(order, env);
 
+    // Clamp unsafe instances where a Func f accesses a Func g using
+    // an index which depends on a third Func h.
+    debug(1) << "Clamping unsafe data-dependent accesses\n";
+    s = clamp_unsafe_accesses(s, env, func_bounds);
+    log("Lowering after clamping unsafe data-dependent accesses", s);
+
     // This pass injects nested definitions of variable names, so we
     // can't simplify statements from here until we fix them up. (We
     // can still simplify Exprs).
@@ -314,7 +311,7 @@ void lower_impl(const vector<Function> &output_funcs,
     log("Lowering after unrolling:", s);
 
     debug(1) << "Vectorizing...\n";
-    s = vectorize_loops(s, env, t);
+    s = vectorize_loops(s, env);
     s = simplify(s);
     log("Lowering after vectorizing:", s);
 
@@ -397,9 +394,15 @@ void lower_impl(const vector<Function> &output_funcs,
     log("Lowering after removing dead allocations and hoisting loop invariants:", s);
 
     debug(1) << "Finding intrinsics...\n";
+    // Must be run after the last simplification, because it turns
+    // divisions into shifts, which the simplifier reverses.
     s = find_intrinsics(s);
     log("Lowering after finding intrinsics:", s);
 
+    debug(1) << "Hoisting prefetches...\n";
+    s = hoist_prefetches(s);
+    log("Lowering after hoisting prefetches:", s);
+
     debug(1) << "Lowering after final simplification:\n"
              << s << "\n\n";
 
@@ -469,8 +472,8 @@ void lower_impl(const vector<Function> &output_funcs,
                 << ", which was not found in the argument list.\n";
 
             err << "\nArgument list specified: ";
-            for (size_t i = 0; i < args.size(); i++) {
-                err << args[i].name << " ";
+            for (const auto &arg : args) {
+                err << arg.name << " ";
             }
             err << "\n\nParameters referenced in generated code: ";
             for (const InferredArgument &ia : inferred_args) {
