@@ -7,7 +7,9 @@
  * pipeline.
  */
 
+#include <initializer_list>
 #include <map>
+#include <memory>
 #include <vector>
 
 #include "ExternalCode.h"
@@ -121,8 +123,8 @@ public:
         }
         template<typename T, typename... Args,
                  typename = typename std::enable_if<Internal::all_are_convertible<Buffer<>, Args...>::value>::type>
-        RealizationArg(Buffer<T> &a, Args &&...args) {
-            buffer_list.reset(new std::vector<Buffer<>>({a, args...}));
+        RealizationArg(Buffer<T> &a, Args &&...args)
+            : buffer_list(std::make_unique<std::vector<Buffer<>>>(std::initializer_list<Buffer<>>{a, std::forward<Args>(args)...})) {
         }
         RealizationArg(RealizationArg &&from) = default;
 
@@ -484,28 +486,8 @@ public:
     const std::vector<CustomLoweringPass> &custom_lowering_passes();
 
     /** See Func::realize */
-    // @{
     Realization realize(std::vector<int32_t> sizes = {}, const Target &target = Target(),
                         const ParamMap &param_map = ParamMap::empty_map());
-    HALIDE_ATTRIBUTE_DEPRECATED("Call realize() with a vector<int> instead")
-    Realization realize(int x_size, int y_size, int z_size, int w_size, const Target &target = Target(),
-                        const ParamMap &param_map = ParamMap::empty_map());
-    HALIDE_ATTRIBUTE_DEPRECATED("Call realize() with a vector<int> instead")
-    Realization realize(int x_size, int y_size, int z_size, const Target &target = Target(),
-                        const ParamMap &param_map = ParamMap::empty_map());
-    HALIDE_ATTRIBUTE_DEPRECATED("Call realize() with a vector<int> instead")
-    Realization realize(int x_size, int y_size, const Target &target = Target(),
-                        const ParamMap &param_map = ParamMap::empty_map());
-
-    // Making this a template function is a trick: `{intliteral}` is a valid scalar initializer
-    // in C++, but we want it to match the vector call, not the (deprecated) scalar one.
-    template<typename T, typename = typename std::enable_if<std::is_same<T, int>::value>::type>
-    HALIDE_ATTRIBUTE_DEPRECATED("Call realize() with a vector<int> instead")
-    HALIDE_ALWAYS_INLINE Realization realize(T x_size, const Target &target = Target(),
-                                             const ParamMap &param_map = ParamMap::empty_map()) {
-        return realize(std::vector<int32_t>{x_size}, target, param_map);
-    }
-    // @}
 
     /** Evaluate this Pipeline into an existing allocated buffer or
      * buffers. If the buffer is also one of the arguments to the
