@@ -48,16 +48,39 @@ enum class TailStrategy {
      * case to handle the if statement. */
     GuardWithIf,
 
-    /** Guard the inner loop with an if statement that prevents
-     * evaluation beyond the original extent, with a hint that the
-     * if statement should be implemented with predicated operations.
-     * Always legal. The if statement is treated like a boundary
-     * condition, and factored out into a loop epilogue if possible.
-     * Pros: no redundant re-evaluation; does not constrain input our
+    /** Guard the loads and stores in the loop with an if statement
+     * that prevents evaluation beyond the original extent. Always
+     * legal. The if statement is treated like a boundary condition,
+     * and factored out into a loop epilogue if possible.
+     * Pros: no redundant re-evaluation; does not constrain input or
      * output sizes. Cons: increases code size due to separate
      * tail-case handling. */
     Predicate,
 
+    /** Guard the loads in the loop with an if statement that
+     * prevents evaluation beyond the original extent. Only legal
+     * for innermost splits. Not legal for RVars, as it would change
+     * the meaning of the algorithm. The if statement is treated like
+     * a boundary condition, and factored out into a loop epilogue if
+     * possible.
+     * Pros: does not constrain input sizes, output size constraints
+     * are simpler than full predication. Cons: increases code size
+     * due to separate tail-case handling, constrains the output size
+     * to be a multiple of the split factor. */
+    PredicateLoads,
+
+    /** Guard the stores in the loop with an if statement that
+     * prevents evaluation beyond the original extent. Only legal
+     * for innermost splits. Not legal for RVars, as it would change
+     * the meaning of the algorithm. The if statement is treated like
+     * a boundary condition, and factored out into a loop epilogue if
+     * possible.
+     * Pros: does not constrain output sizes, input size constraints
+     * are simpler than full predication. Cons: increases code size
+     * due to separate tail-case handling, constraints the input size
+     * to be a multiple of the split factor.. */
+    PredicateStores,
+
     /** Prevent evaluation beyond the original extent by shifting
      * the tail case inwards, re-evaluating some points near the
      * end. Only legal for pure variables in pure definitions. If
