@@ -285,9 +285,11 @@ public:
         check("vpacko(v*.w,v*.w)", hvx_width / 2, in_u16(2 * x + 1));
         check("vdeal(v*,v*,r*)", hvx_width / 4, in_u32(2 * x + 1));
 
-        check("vlut32(v*.b,v*.b,r*)", hvx_width / 1, in_u8(3 * x / 2));
-        check("vlut16(v*.b,v*.h,r*)", hvx_width / 2, in_u16(3 * x / 2));
-        check("vlut16(v*.b,v*.h,r*)", hvx_width / 2, in_u32(3 * x / 2));
+        check("vdelta(v*,v*)", hvx_width / 1, in_u8(3 * x / 2));
+        check("vdelta(v*,v*)", hvx_width / 2, in_u16(3 * x / 2));
+        check("vdelta(v*,v*)", hvx_width / 2, in_u32(3 * x / 2));
+        check("vdelta(v*,v*)", hvx_width * 3, in_u16(x * 3));
+        check("vdelta(v*,v*)", hvx_width * 3, in_u8(x * 3));
 
         check("vlut32(v*.b,v*.b,r*)", hvx_width / 1, in_u8(u8_1));
         check("vlut32(v*.b,v*.b,r*)", hvx_width / 1, in_u8(clamp(u16_1, 0, 63)));
@@ -296,11 +298,6 @@ public:
         check("vlut16(v*.b,v*.h,r*)", hvx_width / 2, in_u32(u8_1));
         check("vlut16(v*.b,v*.h,r*)", hvx_width / 2, in_u32(clamp(u16_1, 0, 15)));
 
-        // This tests the case of vlut with > 256 elements (thus forcing us to split into
-        // multiple vluts)
-        check("vlut16(v*.b,v*.h,r*)", hvx_width * 3, in_u16(x * 3));
-        check("vlut32(v*.b,v*.b,r*)", hvx_width * 3, in_u8(x * 3));
-
         check("v*.ub = vpack(v*.h,v*.h):sat", hvx_width / 1, u8_sat(i16_1));
         check("v*.b = vpack(v*.h,v*.h):sat", hvx_width / 1, i8_sat(i16_1));
         check("v*.uh = vpack(v*.w,v*.w):sat", hvx_width / 2, u16_sat(i32_1));
@@ -613,19 +610,16 @@ public:
         check("vrmpy(v*.ub,v*.b)", hvx_width, i16(u8_1)*i8_1 + i16(u8_2)*i8_2 + i16(u8_3)*i8_3 + i16(u8_4)*i8_4);
 #endif
 
-#if 0
-        // Temporarily disabling this vrmpy test because of https://github.com/halide/Halide/issues/4248
         // These should also work with 16 bit results. However, it is
         // only profitable to do so if the interleave simplifies away.
         Expr u8_4x4[] = {
-            in_u8(4*x + 0),
-            in_u8(4*x + 1),
-            in_u8(4*x + 2),
-            in_u8(4*x + 3),
+            in_u8(4 * x + 0),
+            in_u8(4 * x + 1),
+            in_u8(4 * x + 2),
+            in_u8(4 * x + 3),
         };
-        check("vrmpy(v*.ub,r*.b)", hvx_width/2, i16(u8_4x4[0])*127 + i16(u8_4x4[1])*126 + i16(u8_4x4[2])*-125 + i16(u8_4x4[3])*124);
+        check("vrmpy(v*.ub,r*.b)", hvx_width / 2, i16(u8_4x4[0]) * 127 + i16(u8_4x4[1]) * 126 + i16(u8_4x4[2]) * -125 + i16(u8_4x4[3]) * 124);
 
-#endif
         // Make sure it doesn't generate if the operands don't interleave.
         check("vmpa(v*.ub,r*.b)", hvx_width, i16(u8_1) * 127 + i16(u8_2) * -126 + i16(u8_3) * 125 + i16(u8_4) * 124);
 
